// The "freEvolve System" is a platform supporting the reconfiguration of 
// component-based applications'
// Copyright (C) 1999  Ralph Hinken and Dr. Oliver Stiemerling
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Version: 1.0
// Date:    09.10.2001
//
// History:
//

package org.freevolve.server;
import java.util.*;
import java.rmi.*;
import java.rmi.server.*;

/**
    Reprsentiert einen Parameter einer abstrakten Komponente.<br><br>
    Der Parameter muss extra gesetzt werden und der Type korrekt
    angegeben werden. Mglich sind null, boolean, String, int,
    double und char.<br><br>
    
    Der Parameter ist letztentlich dafr da, einen Parameter in einer
    FlexiBean zu setzen. <br><br>

    Entweder stellt dieses Param Objekt einen Parameter dar, der ber
    ParamBinding Objekte nach oben in der Komponentenhierarchie
    weitergereicht werden kann. Oder er setzt einen Parameter einer
    Subkomponente auf einen Wert. In diesem Fall wird der lokale Name der
    Subkomponente gemerkt und der Flag isSubCompParam ist "true".
    
    @see Component
    @see ParamBinding
    */
public class Param extends UnicastRemoteObject implements IR_Param {
  private String    name;
  private String    type;
  private String    svalue; // null, boolean, String;
  private double    nvalue; // int, double;
  private char      cvalue; // char;
  private Component owner;
  private Vector    bindings; // IR_ParamBinding
  private boolean   hasValue;
  private  String   subComponentInstanceName;
  private boolean   isSubCompParam;
  public  IR_Param  settingParam;

  public Param(String name, String type, String localName, Component owner)  throws RemoteException {
    bindings = new Vector();
    this.name=name;
    this.type = type;
    svalue    = null;
    nvalue    = 0.0;
    cvalue    = '?';
    this.owner = owner;
    hasValue = false;
    subComponentInstanceName = localName;
    if (localName != null) isSubCompParam = true;
    else                   isSubCompParam = false;
    settingParam = null;

  }

   /** Gibt dem lokalen Namen der Subkomponente zurck.
        Das ist die Subkomponente, deren Parameter gesetzt wurde.
   */
   public String getSubCompName() { return subComponentInstanceName; }

   /**  Setzt den lokalen Namen der Subkomponente deren Parameter belegt werden soll. 
        Somit steht dieser Parameter fr die Wertbelegung eines Parameters der
        Subkomponente "localName".
   */
   public void setSubCompName(String localName) {
         subComponentInstanceName = localName; 
         isSubCompParam = true;
   }

   /** Setzt die Referenz auf den Wertbelegungsparameter der Superkomponente, 
      der den Wert dieses Parameters bestimmt.
   */
   public void setSuperParam(IR_Param p) {settingParam = p; }

   /** Gibt den Parameter der Superkomponente zurck, der die Wertbelegung dieses Parameters setzt.
   */
   public IR_Param getSuperParam() { return settingParam; }

   /** Testet, ob dieser Parameter den Wert eines Parameters einer Subkomponente belegt.
   */
   public boolean isSubCompParam() { return isSubCompParam;}

   /** Setzt den Namen des Parameters (neu). 
   */
   public void setName(String name) {
      this.name = name;
      if (settingParam != null) {
         try { settingParam.setName(name); } catch (RemoteException e) {
            System.err.println(e); }
      }
   }


  /** Gibt es eine Wertbelegung der Superkomponente?
  */
  public boolean hasSuperValue() {
      if (settingParam==null) return false;
      else return true;
  }

  /** Wurde der Parameter auf einen Wert gesetzt */
  public boolean isValueSet () {return  hasValue;}

  /** Markiert Parameter als ohne Wertbelegung */
  public void unSetValue() { 
      svalue=null;
      nvalue=0.0;
      cvalue='?';
      hasValue=false; 
  }
  /** Setzt einen String-Wert der Typen "null", boolean oder String
   */
  public void setValue(String value) { svalue = value; hasValue=true; }
  /** Setzt einen numerischen Wert der Typen int oder double */
  public void setValue(double value) { nvalue = value; hasValue=true;}
  /** Setzt einen Character-Wert */
  public void setValue(char   value) { cvalue = value; hasValue=true; }
   
  /** Setzt den Typ des Parameters. Wird im Konstruktor mit bergeben,
      kann aber nachtrglich verndert werden. Wird von comp_io benutzt.
      @see comp_io18.03.99
  */
  public void setType(String type) { this.type = type;}
  /** Gibt einen String-Wert zurck, der noch interpretiert werden muss.
    Je nach Type null, boolean oder String.
    @return null, falls nicht gesetzt.
    @see #getType
    */
  public String getSValue() { 
      if (hasValue) return svalue; 
      else return null;
   }
  /** Gibt einen numerischen Wert zurck, der noch interpretiert werden muss.
    Je nach Type int oder double.
    @return 0.0, falls nicht gesetzt.
    @see #getType
    */
  public double getNValue() { if (hasValue) return nvalue; else return 0.0;}
    /** Gibt einen char-Wert zurueck.
        @return '?', falls nicht gesetzt.
        @see #getType
    */
  public char   getCValue() {if (hasValue)  return cvalue; else return '?' ;}

  public Object getValue() {
    if (hasSuperValue()) 
      try { return settingParam.getValue();
      } catch (RemoteException e) {
         try {owner.getServer().message("error", 1, "Param.getValue: " + e.toString()); }
         catch (RemoteException d) {}
      }
    if (! hasValue) return null;
    if (svalue != null) {
      if (type.equalsIgnoreCase("boolean")) {
          if (svalue.equalsIgnoreCase("true")) return (Object) new Boolean(true);
          else return (Object) new Boolean(false);
      }
      if (svalue.equalsIgnoreCase("null")) return null;
      return (Object) svalue;
    }
    if (nvalue != 0.0) return (Object) new Double(nvalue);
    if (cvalue != '?') return (Object) new Character(cvalue);
    return null;
  }

  /** Informiert ber den Type des dargestellten Parameters.
    Dies ist null, boolean, String, int, double oder char
    */
  public String getType() { return type; }

  /** Der Name des Parameters.
  */
   public String getName() { return name; }
  /** Debug Informationen ber den Parameter */
  public String print() {
    return ("Param "+ name + " t " + type + " s=" + svalue + " n=" + nvalue + " c=" +cvalue);
  }

  public String printShort() {
    String result;
    result = "pa " + name + " " + type;
    if (isValueSet() || hasSuperValue())  result += " V " + getValue().toString();
    else result += " N";
    return result;
  }

   /** Der Besitzer des Parameters.
   */
   public IR_Component getOwner() { return owner; }

  /** Anmelden einer lokalen Verbindung.
    Diese Verbindung geht zu einem Param entweder (1) einer Subkomponente oder (2) des Vorgngers unserer Owner-Komponente.
    (1) und (2) sind Weiterrichungen eines Parameters.
    */
  public void addBinding(IR_ParamBinding p) {
    if (p==null) return;
    bindings.addElement(p);
  }

  /** Abmelden einer lokalen Verbindung.
    Diese Verbindung geht zu einem Param entweder (1) einer Subkomponente oder (2) des Vorgngers unserer Owner-Komponente.
    (1) und (2) sind Weiterrichungen eines Parameters.
    */
  public void removeBinding(IR_ParamBinding p) {
    if (p==null) return;
    bindings.removeElement(p);
  }

  /** Gibt alle lokalen Verbindunge an diesen Port zurck. Keine entfernten Verbindungen.
    @return Vector      von IR_ParamBinding
    @see ParamBinding
    */
  public Vector      getBindings() {
    return bindings;
  }

  /** Testet, ob dieser Parameter mit der Komponente comp verbunden ist.
      Ruft ParamBinding.connects(comp, this.owner) auf.
      @see ParamBinding#connects
  */
  public boolean isConnected(IR_Component comp) {
      IR_ParamBinding pb = null;
      try {
         for( Enumeration e= bindings.elements(); e.hasMoreElements() ; ) {
             pb = (IR_ParamBinding) e.nextElement();  
             if (pb.connects(comp, owner)) return true;
         }
      } catch (RemoteException e) {
         try {owner.getServer().message("error", 1, "Param.isConnected: " + e.toString()); }
         catch (RemoteException d) {}
      }
      return false;
  }

}