// The "freEvolve System" is a platform supporting the reconfiguration of 
// component-based applications'
// Copyright (C) 1999  Ralph Hinken and Dr. Oliver Stiemerling
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Version: 1.0
// Date:    09.10.2001
//
// History:
//

package org.freevolve.server;
import java.util.*;
import java.rmi.*;
import java.rmi.server.*;
import java.lang.reflect.*;

// for intern use only
/** Diese Klasse reprsentiert einen Port einer Bean/Komponente.
  Entweder ist dies ein Port einer FlexiBean oder ein hochgereichter
  Port innerhalb der Komponentenhierarchie. <br><br>
  Ein Port hat einen Namen, der innerhalb der Komponente eindeutig ist.
  Er hat einen Typ, eine Polaritt (provided, required, symmetric), die
  bestimmt, ob der Port ein Eingabe- oder Ausgabeport ist, und ein
  Protokoll (event oder sharedObject) dass den Interaktionstyp definiert. <br><br>
  Der Port merkt sich die Komponente, von der er Port ist und alle
  PortBinding Objekte, die diesen Port verbinden. <b>Der Port merkt sich
  nicht die Remote Verbindungen - die kennt er nicht.</b>
  @see Component
  @see PortBinding
  */
public class Port  extends UnicastRemoteObject implements IR_Port {
  static final int P_PROVIDED = -1;
  static final int P_REQUIRED = -2;
  static final int P_SYMMETRIC = -3;
  static final int P_UNDEFINED = -4;
  String    name;
  String    type;
  int       polarity;  // provided, required, symmetric
  Component owner;
  Vector    bindings;
  String    protocol;
  boolean   event;
  /*
  public Port()  throws RemoteException {
    super();
    bindings = new Vector();
  }
  */

  public Port(String name, String type, String polarity, String protocol, Component owner)  throws RemoteException {
    super();
    this.name = name;
    this.type = type;
    setPolarity(polarity);
    this.owner = owner;
    bindings = new Vector();
    if (protocol.equalsIgnoreCase("event")) this.protocol = "event";
    else if (protocol.equalsIgnoreCase("sharedObject")) this.protocol = "sharedObject";    
    else this.protocol = "undefined";
  }

   /** Setzt den Namen des Ports. 
   */
   public void setName(String name) {
      this.name = name;
    
   }

   /** Setzt den Interaktionstyp nachtrglich.
      @param  protocol "event", "sharedObject" oder "undefined"
   */
   public void setProtocol(String protocol) {
      if (protocol == null) return;
      if (protocol.equalsIgnoreCase("event")) this.protocol = "event";
      else if (protocol.equalsIgnoreCase("sharedObject")) this.protocol = "sharedObject";    
      else this.protocol = "undefined";
  }

  /**   Der Name des Ports, der bezglich der owner Komponente eindeutig ist.
   */
  public String getName() { return name; }

  /**   Der Typ des Ports.
    Er wird benutzt, um zwei Ports auf gleichen Typ hin zu untersuchen
   */
  public String getType() { return type; }

  /** Die Komponente, die diesen Port besitzt und erzeugt hat.
   */
  public IR_Component  getOwner() {return owner; }

  /*+ Info ber den Port mit Name, Typ, Polaritt
   */
  public String print() {
    return ("Port: name=" + this.name + " type=" + this.type + " polarity=" + this.getPolarity() + " protocol=" + this.getProtocol());
  }

  public String printShort() {
    return "po " + this.getPolarity() + " "+this.name + " " + this.type;
  }

  /** Anmelden einer lokalen Verbindung.
    Diese Verbindung geht zu einem Port entweder (1) einer Subkomponente oder (2) der Subkomponente des Vorgngers
    unserer Owner-Komponente innerhalb der Komponentenhierarchie oder (3) des Vorgngers unserer Owner-Komponente.
    (1) und (3) sind Weiterrichungen eines Ports, (2) ist eine Querrverbindung, eine Verbindung von zwei Ports.
    */
  public void addBinding(IR_PortBinding p) {
    if (p==null) return;
    bindings.addElement(p);
  }

  /** Abmelden einer lokalen Verbindung.
    Diese Verbindung ging zu einem Port entweder (1) einer Subkomponente oder (2) der Subkomponente des Vorgngers
    unserer Owner-Komponente innerhalb der Komponentenhierarchie oder (3) des Vorgngers unserer Owner-Komponente.
    (1) und (3) sind Weiterrichungen eines Ports, (2) ist eine Querrverbindung, eine Verbindung von zwei Ports.
    */
  public void removeBinding(IR_PortBinding p) {
    if (p==null) return;
    bindings.removeElement(p);
  }

  /** Gibt alle lokalen Verbindunge an diesen Port zurck. Keine entfernten Verbindungen.
    @return Vector      von IR_PortBinding
    @see PortBinding
    */
  public Vector      getBindings() {
    return bindings;
  }

  /** Setzt die Polaritt des Ports. Provided, Required oder Symmetric.
    @param p String, der entweder Provided, Required oder Symmetric enthlt
    */
  public void setPolarity(String p) {
    switch (p.charAt(0)) {
      case 'p': case 'P': this.polarity = P_PROVIDED; break;
      case 'r': case 'R': this.polarity = P_REQUIRED; break;
      case 's': case 'S': this.polarity = P_SYMMETRIC; break;
      default: this.polarity = P_UNDEFINED;
    }
  }
  
  /** Die Polaritt des Ports. Provided, Required oder Symmetric.
    @return  String, der entweder provided, required, symmetric oder undefined enthlt.
    */
  public String getPolarity() {
    switch (this.polarity) {
    case -1: return "provided"; 
    case -2: return "required";
    case -3: return "symmetric";
    default: return "undefined";
    }
  }

  /** Testet, ob dieser Port mit der Komponente comp verbunden ist.
      Ruft PortBinding.connects(comp, this.owner) auf.
      @see PortBinding#connects
  */
  public boolean isConnected(IR_Component comp) {
      IR_PortBinding pb = null;
      try {
         for( Enumeration e= bindings.elements(); e.hasMoreElements() ; ) {
             pb = (IR_PortBinding) e.nextElement();  
             if (pb.connects(comp, owner)) return true;
         }
      } catch (RemoteException e) {
         try {owner.getServer().message("error", 1, "Port.isConnected: " + e.toString()); }
         catch (RemoteException d) {}
      }
      return false;
 }

 /** Gibt den Interaktionstyp zurck. 
     @return event oder sharedObject oder undefined
 */
 public String    getProtocol() {
   return protocol;
 }
 
 public boolean readFuctions(Class klasse) {
    Method m[] = klasse.getDeclaredMethods();
    //owner.message("i",1,toTxt());                                                
    if (getPolarity().equals("provided")) {
       String postfix = type + "_" + name;
       String addListener = "add" + postfix;
       String getShared = "getShared" + postfix;
       String removeListener ="remove" + postfix;
       String methodName = null;
   //    owner.message("i",1,addListener+"\n"+getShared+"\n"+removeListener);                           
       for(int i=0; i<m.length;i++){ 
          methodName = m[i].getName();
          //owner.message("info",1,m[i].getReturnType().getName() + " aus " + m[i].getName());
          if (addListener.equals(methodName)) {
               //add = m[i];
               event = true;
               owner.message("info",1,m[i].getReturnType().getName() + " aus " + m[i].getName());
               Class pt[] = m[i].getParameterTypes();
               if (pt.length>0) owner.message("i",1,pt[0].getName());
         } else if (getShared.equals(methodName)) {
               //get = m[i];
               event =false;
               owner.message("info",1,m[i].getReturnType().getName() + " aus " + m[i].getName());
               Class pt[] = m[i].getParameterTypes();
               if (pt.length>0) owner.message("i",1,pt[0].getName());
         } else if (removeListener.equals(methodName)) {
               //remove = m[i];
               event = true;
               owner.message("info",1,m[i].getReturnType().getName() + " aus " + m[i].getName());
               Class pt[] = m[i].getParameterTypes();
               if (pt.length>0) owner.message("i",1,pt[0].getName());
         }
      }
      //" von " + (m[i].getParameterTypes())[0].getName());
    } else if (getPolarity().equals("required")) {
       String postfix = type + "_" + name;
       String getListener = "getPort" + postfix;
       String setShared = "setShared" + postfix;
       String forgetShared ="forgetShared" + postfix;
       String methodName = null;
   //    owner.message("i",1,addListener+"\n"+getShared+"\n"+removeListener);
       for(int i=0; i<m.length;i++){ 
          methodName = m[i].getName();
          //owner.message("info",1,m[i].getReturnType().getName() + " aus " + m[i].getName());
          if (getListener.equals(methodName)) {
               //get = m[i];
               event = true;
               owner.message("info",1,m[i].getReturnType().getName() + " aus " + m[i].getName());
               Class pt[] = m[i].getParameterTypes();
               if (pt.length>0) owner.message("i",1,pt[0].getName());
         } else if (setShared.equals(methodName)) {
               //add = m[i];
               event = false;
               owner.message("info",1,m[i].getReturnType().getName() + " aus " + m[i].getName());
               Class pt[] = m[i].getParameterTypes();
               if (pt.length>0) owner.message("i",1,pt[0].getName());
         } else if (forgetShared.equals(methodName)) {
               //remove = m[i];
               event = false;
               owner.message("info",1,m[i].getReturnType().getName() + " aus " + m[i].getName());
               Class pt[] = m[i].getParameterTypes();
               if (pt.length>0) owner.message("i",1,pt[0].getName());
         }
       }

    }
    return true;
  }

      
}
